@@ -1,7 +1,7 @@
 /* beta.c: Routines for handling the Beta disk interface
    Copyright (c) 2004-2008 Stuart Brady
 
-   $Id: beta.c 3681 2008-06-16 09:40:29Z pak21 $
+   $Id: beta.c 4180 2010-10-09 12:59:37Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -53,11 +53,23 @@
 #include "wd_fdc.h"
 #include "z80/z80.h"
 #include "z80/z80_macros.h"
+#include "options.h"	/* needed for get combo options */
+
+#define DISK_TRY_MERGE(heads) ( option_enumerate_diskoptions_disk_try_merge() == 2 || \
+				( option_enumerate_diskoptions_disk_try_merge() == 1 && heads == 1 ) )
+
+/* Two 8Kb memory chunks accessible by the Z80 when /ROMCS is low */
+memory_page beta_memory_map_romcs[2];
 
 int beta_available = 0;
 int beta_active = 0;
 int beta_builtin = 0;
 
+static libspectrum_byte beta_system_register; /* FDC system register */
+
+libspectrum_word beta_pc_mask;
+libspectrum_word beta_pc_value;
+
 static int beta_index_pulse = 0;
 
 static int index_event;
@@ -117,8 +129,8 @@ beta_memory_map( void )
 {
   if( !beta_active ) return;
 
-  memory_map_read[0] = memory_map_write[0] = memory_map_romcs[ 0 ];
-  memory_map_read[1] = memory_map_write[1] = memory_map_romcs[ 1 ];
+  memory_map_read[0] = memory_map_write[0] = beta_memory_map_romcs[0];
+  memory_map_read[1] = memory_map_write[1] = beta_memory_map_romcs[1];
 }
 
 static void
@@ -137,13 +149,13 @@ beta_init( void )
 {
   int i;
   wd_fdc_drive *d;
-  
+
   beta_fdc = wd_fdc_alloc_fdc( FD1793, 0, WD_FLAG_BETA128 );
   beta_fdc->current_drive = NULL;
 
   for( i = 0; i < BETA_NUM_DRIVES; i++ ) {
     d = &beta_drives[ i ];
-    fdd_init( &d->fdd, FDD_SHUGART, 0, 0 );	/* drive geometry 'autodetect' */
+    fdd_init( &d->fdd, FDD_SHUGART, NULL, 0 );	/* drive geometry 'autodetect' */
     d->disk.flag = DISK_FLAG_NONE;
   }
   beta_select_drive( 0 );
@@ -158,6 +170,7 @@ beta_init( void )
   if( index_event == -1 ) return 1;
 
   module_register( &beta_module_info );
+  for( i = 0; i < 2; i++ ) beta_memory_map_romcs[i].bank = MEMORY_BANK_ROMCS;
 
   return 0;
 }
@@ -167,6 +180,7 @@ beta_reset( int hard_reset GCC_UNUSED )
 {
   int i;
   wd_fdc_drive *d;
+  const fdd_params_t *dt;
 
   event_remove_type( index_event );
 
@@ -178,6 +192,9 @@ beta_reset( int hard_reset GCC_UNUSED )
 
   beta_available = 1;
 
+  beta_pc_mask = 0xff00;
+  beta_pc_value = 0x3d00;
+  
   wd_fdc_master_reset( beta_fdc );
 
   for( i = 0; i < BETA_NUM_DRIVES; i++ ) {
@@ -188,37 +205,75 @@ beta_reset( int hard_reset GCC_UNUSED )
   }
 
   if( !beta_builtin ) {
-    machine_load_rom_bank( memory_map_romcs, 0, 0,
+    machine_load_rom_bank( beta_memory_map_romcs, 0, 0,
 			   settings_current.rom_beta128,
 			   settings_default.rom_beta128, 0x4000 );
 
-    memory_map_romcs[ 0 ].writable = 0;
-    memory_map_romcs[ 1 ].writable = 0;
+    beta_memory_map_romcs[ 0 ].writable = 0;
+    beta_memory_map_romcs[ 1 ].writable = 0;
 
-    memory_map_romcs[0].source = MEMORY_SOURCE_PERIPHERAL;
-    memory_map_romcs[1].source = MEMORY_SOURCE_PERIPHERAL;
+    beta_memory_map_romcs[0].source = MEMORY_SOURCE_PERIPHERAL;
+    beta_memory_map_romcs[1].source = MEMORY_SOURCE_PERIPHERAL;
 
     beta_active = 0;
+
+    if( !( machine_current->capabilities &
+           LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) ) {
+      beta_pc_mask = 0xfe00;
+      beta_pc_value = 0x3c00;
+
+      /* For 48K type machines, the Beta 128 is supposed to be configured
+         to start with the Beta ROM paged in (System switch in centre position)
+         */
+      beta_page();
+    }
   }
 
   /* We can eject disks only if they are currently present */
+  dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128a_type() + 1 ];	/* +1 => there is no `Disabled' */
+  fdd_init( &beta_drives[ BETA_DRIVE_A ].fdd, FDD_SHUGART, dt, 1 );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A, dt->enabled );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_EJECT,
 		    beta_drives[ BETA_DRIVE_A ].fdd.loaded );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_FLIP_SET,
+		    !beta_drives[ BETA_DRIVE_A ].fdd.upsidedown );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_WP_SET,
 		    !beta_drives[ BETA_DRIVE_A ].fdd.wrprot );
+
+
+  dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128b_type() ];
+  fdd_init( &beta_drives[ BETA_DRIVE_B ].fdd, dt->enabled ? FDD_SHUGART : FDD_TYPE_NONE, dt, 1 );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B, dt->enabled );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_FLIP_SET,
+		    !beta_drives[ BETA_DRIVE_B ].fdd.upsidedown );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_EJECT,
 		    beta_drives[ BETA_DRIVE_B ].fdd.loaded );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_WP_SET,
 		    !beta_drives[ BETA_DRIVE_B ].fdd.wrprot );
+
+
+  dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128c_type() ];
+  fdd_init( &beta_drives[ BETA_DRIVE_C ].fdd, dt->enabled ? FDD_SHUGART : FDD_TYPE_NONE, dt, 1 );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C, dt->enabled );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_FLIP_SET,
+		    !beta_drives[ BETA_DRIVE_C ].fdd.upsidedown );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_EJECT,
 		    beta_drives[ BETA_DRIVE_C ].fdd.loaded );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_WP_SET,
 		    !beta_drives[ BETA_DRIVE_C ].fdd.wrprot );
+
+
+  dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128d_type() ];
+  fdd_init( &beta_drives[ BETA_DRIVE_D ].fdd, dt->enabled ? FDD_SHUGART : FDD_TYPE_NONE, dt, 1 );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D, dt->enabled );
+  ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_FLIP_SET,
+		    !beta_drives[ BETA_DRIVE_D ].fdd.upsidedown );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_EJECT,
 		    beta_drives[ BETA_DRIVE_D ].fdd.loaded );
   ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_WP_SET,
 		    !beta_drives[ BETA_DRIVE_D ].fdd.wrprot );
 
+
   beta_select_drive( 0 );
   machine_current->memory_map();
   beta_event_index( 0, 0, NULL );
@@ -312,6 +367,8 @@ beta_sp_write( libspectrum_word port GCC_UNUSED, libspectrum_byte b )
   fdd_set_head( &beta_fdc->current_drive->fdd, ( ( b & 0x10 ) ? 0 : 1 ) );
   /* 0x20 = density, reset = FM, set = MFM */
   beta_fdc->dden = b & 0x20 ? 1 : 0;
+
+  beta_system_register = b;
 }
 
 libspectrum_byte
@@ -342,6 +399,7 @@ beta_disk_insert( beta_drive_number which, const char *filename,
 {
   int error;
   wd_fdc_drive *d;
+  const fdd_params_t *dt;
 
   if( which >= BETA_NUM_DRIVES ) {
     ui_error( UI_ERROR_ERROR, "beta_disk_insert: unknown drive %d",
@@ -358,14 +416,29 @@ beta_disk_insert( beta_drive_number which, const char *filename,
   }
 
   if( filename ) {
-    error = disk_open( &d->disk, filename, 0 );
+    error = disk_open( &d->disk, filename, 0, DISK_TRY_MERGE( d->fdd.fdd_heads ) );
     if( error != DISK_OK ) {
       ui_error( UI_ERROR_ERROR, "Failed to open disk image: %s",
 				disk_strerror( d->disk.status ) );
       return 1;
     }
   } else {
-    error = disk_new( &d->disk, 2, 80, DISK_DENS_AUTO, DISK_UDI );
+    switch( which ) {
+    case 0:
+      dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128a_type() + 1 ];	/* +1 => there is no `Disabled' */
+      break;
+    case 1:
+      dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128b_type() ];
+      break;
+    case 2:
+      dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128c_type() ];
+      break;
+    case 3:
+    default:
+      dt = &fdd_params[ option_enumerate_diskoptions_drive_beta128d_type() ];
+      break;
+    }
+    error = disk_new( &d->disk, dt->heads, dt->cylinders, DISK_DENS_AUTO, DISK_UDI );
     if( error != DISK_OK ) {
       ui_error( UI_ERROR_ERROR, "Failed to create disk image: %s",
 				disk_strerror( d->disk.status ) );
@@ -379,21 +452,29 @@ beta_disk_insert( beta_drive_number which, const char *filename,
   switch( which ) {
   case BETA_DRIVE_A:
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_EJECT, 1 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_A ].fdd.upsidedown );
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_WP_SET,
 		      !beta_drives[ BETA_DRIVE_A ].fdd.wrprot );
     break;
   case BETA_DRIVE_B:
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_EJECT, 1 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_B ].fdd.upsidedown );
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_WP_SET,
 		      !beta_drives[ BETA_DRIVE_B ].fdd.wrprot );
     break;
   case BETA_DRIVE_C:
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_EJECT, 1 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_C ].fdd.upsidedown );
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_WP_SET,
 		      !beta_drives[ BETA_DRIVE_C ].fdd.wrprot );
     break;
   case BETA_DRIVE_D:
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_EJECT, 1 );
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_D ].fdd.upsidedown );
     ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_WP_SET,
 		      !beta_drives[ BETA_DRIVE_D ].fdd.wrprot );
     break;
@@ -408,6 +489,43 @@ beta_disk_insert( beta_drive_number which, const char *filename,
   return 0;
 }
 
+int
+beta_disk_flip( beta_drive_number which, int flip )
+{
+  wd_fdc_drive *d;
+
+  if( which >= BETA_NUM_DRIVES )
+    return 1;
+
+  d = &beta_drives[ which ];
+
+  if( !d->fdd.loaded )
+    return 1;
+
+  fdd_flip( &d->fdd, flip );
+
+  /* Set the 'flip' item */
+  switch( which ) {
+  case BETA_DRIVE_A:
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_A_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_A ].fdd.upsidedown );
+    break;
+  case BETA_DRIVE_B:
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_B_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_B ].fdd.upsidedown );
+    break;
+  case BETA_DRIVE_C:
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_C_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_C ].fdd.upsidedown );
+    break;
+  case BETA_DRIVE_D:
+    ui_menu_activate( UI_MENU_ITEM_MEDIA_DISK_BETA_D_FLIP_SET,
+		      !beta_drives[ BETA_DRIVE_D ].fdd.upsidedown );
+    break;
+  }
+  return 0;
+}
+
 int
 beta_disk_writeprotect( beta_drive_number which, int wrprot )
 {
@@ -446,25 +564,28 @@ beta_disk_writeprotect( beta_drive_number which, int wrprot )
 }
 
 int
-beta_disk_eject( beta_drive_number which, int write )
+beta_disk_eject( beta_drive_number which, int saveas )
 {
   wd_fdc_drive *d;
   char drive;
-  
+
   if( which >= BETA_NUM_DRIVES )
     return 1;
-    
+
   d = &beta_drives[ which ];
-  
+
   if( !d->fdd.loaded )
     return 0;
-    
-  if( write ) {
-  
-    if( ui_beta_disk_write( which ) ) return 1;
+
+  if( saveas ) {	/* 1 -> save as.., 2 -> save */
+
+    if( d->disk.filename == NULL ) saveas = 1;
+    if( ui_beta_disk_write( which, 2 - saveas ) ) return 1;
+    d->disk.dirty = 0;
+    return 0;
 
   } else {
-  
+
     if( d->disk.dirty ) {
       ui_confirm_save_t confirm;
 
@@ -485,7 +606,7 @@ beta_disk_eject( beta_drive_number which, int write )
       switch( confirm ) {
 
       case UI_CONFIRM_SAVE_SAVE:
-	if( ui_beta_disk_write( which ) ) return 1;
+	if( beta_disk_eject( which, 2 ) ) return 1;	/* first save */
 	break;
 
       case UI_CONFIRM_SAVE_DONTSAVE: break;
@@ -523,6 +644,7 @@ beta_disk_write( beta_drive_number which, const char *filename )
   int error;
 
   d->disk.type = DISK_TYPE_NONE;
+  if( filename == NULL ) filename = d->disk.filename;	/* write over original file */
   error = disk_write( &d->disk, filename );
 
   if( error != DISK_OK ) {
@@ -531,9 +653,19 @@ beta_disk_write( beta_drive_number which, const char *filename )
     return 1;
   }
 
+  if( d->disk.filename && strcmp( filename, d->disk.filename ) ) {
+    free( d->disk.filename );
+    d->disk.filename = strdup( filename );
+  }
   return 0;
 }
 
+fdd_t *
+beta_get_fdd( beta_drive_number which )
+{
+  return &( beta_drives[ which ].fdd );
+}
+
 static void
 beta_event_index( libspectrum_dword last_tstates, int type, void *user_data )
 {
@@ -582,12 +714,17 @@ beta_from_snapshot( libspectrum_snap *snap )
   if( libspectrum_snap_beta_custom_rom( snap ) &&
       libspectrum_snap_beta_rom( snap, 0 ) &&
       machine_load_rom_bank_from_buffer(
-                             memory_map_romcs, 0, 0,
+                             beta_memory_map_romcs, 0, 0,
                              libspectrum_snap_beta_rom( snap, 0 ),
                              MEMORY_PAGE_SIZE * 2,
                              1 ) )
     return;
 
+  /* ignore drive count for now, there will be an issue with loading snaps where
+     drives have been disabled
+  libspectrum_snap_beta_drive_count( snap )
+   */
+
   beta_fdc->direction = libspectrum_snap_beta_direction( snap );
 
   beta_cr_write ( 0x001f, 0 );
@@ -600,15 +737,15 @@ beta_from_snapshot( libspectrum_snap *snap )
 void
 beta_to_snapshot( libspectrum_snap *snap )
 {
-  int attached;
   wd_fdc *f = beta_fdc;
   libspectrum_byte *buffer;
+  int drive_count = 0;
 
   if( !periph_beta128_active ) return;
 
   libspectrum_snap_set_beta_active( snap, 1 );
 
-  if( memory_map_romcs[0].source == MEMORY_SOURCE_CUSTOMROM ) {
+  if( beta_memory_map_romcs[0].source == MEMORY_SOURCE_CUSTOMROM ) {
     size_t rom_length = MEMORY_PAGE_SIZE * 2;
 
     buffer = malloc( rom_length );
@@ -617,19 +754,25 @@ beta_to_snapshot( libspectrum_snap *snap )
       return;
     }
 
-    memcpy( buffer, memory_map_romcs[0].page, MEMORY_PAGE_SIZE );
-    memcpy( buffer + MEMORY_PAGE_SIZE, memory_map_romcs[1].page,
+    memcpy( buffer, beta_memory_map_romcs[0].page, MEMORY_PAGE_SIZE );
+    memcpy( buffer + MEMORY_PAGE_SIZE, beta_memory_map_romcs[1].page,
 	    MEMORY_PAGE_SIZE );
 
     libspectrum_snap_set_beta_rom( snap, 0, buffer );
     libspectrum_snap_set_beta_custom_rom( snap, 1 );
   }
 
+  drive_count++; /* Drive A is not removable */
+  if( option_enumerate_diskoptions_drive_beta128b_type() > 0 ) drive_count++;
+  if( option_enumerate_diskoptions_drive_beta128c_type() > 0 ) drive_count++;
+  if( option_enumerate_diskoptions_drive_beta128d_type() > 0 ) drive_count++;
+  libspectrum_snap_set_beta_drive_count( snap, drive_count );
+
   libspectrum_snap_set_beta_paged ( snap, beta_active );
   libspectrum_snap_set_beta_direction( snap, beta_fdc->direction );
-  libspectrum_snap_set_beta_status( snap, beta_sr_read( 0x001f, &attached ) );
+  libspectrum_snap_set_beta_status( snap, f->status_register );
   libspectrum_snap_set_beta_track ( snap, f->track_register );
   libspectrum_snap_set_beta_sector( snap, f->sector_register );
   libspectrum_snap_set_beta_data  ( snap, f->data_register );
-  libspectrum_snap_set_beta_system( snap, beta_sp_read( 0x00ff, &attached ) );
+  libspectrum_snap_set_beta_system( snap, beta_system_register );
 }
