@@ -1,7 +1,7 @@
 /* fuse.h: Variables exported from the main file
    Copyright (c) 2000-2003 Philip Kendall
 
-   $Id: fuse.h,v 1.16 2004/01/02 12:05:51 pak21 Exp $
+   $Id: fuse.h 2889 2007-05-26 17:45:08Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -28,22 +27,25 @@
 #define FUSE_FUSE_H
 
 #include <limits.h>
+
 #include <libspectrum.h>
 
+#include "compat.h"
+
 extern char *fuse_progname;		/* argv[0] */
 extern char fuse_directory[ PATH_MAX ];	/* The directory we started in */
 
-extern char* fuse_progname;		/* argv[0] */
-
 extern int fuse_exiting;		/* Shall we exit now? */
 
 extern int fuse_emulation_paused;	/* Is Spectrum emulation paused? */
 int fuse_emulation_pause(void);		/* Stop and start emulation */
 int fuse_emulation_unpause(void);
 
-int fuse_abort( void );			/* Emergency shutdown */
+#ifdef UI_WIN32
+int fuse_main(int argc, char **argv);
+#endif
 
-extern int fuse_sound_in_use;		/* Are we trying to produce sound? */
+void fuse_abort( void ) GCC_NORETURN;	/* Emergency shutdown */
 
 extern libspectrum_creator *fuse_creator; /* Creator information for file
 					     formats which support this */
