@@ -1,7 +1,7 @@
 /* scld.h: Routines for handling the Timex SCLD
-   Copyright (c) 2002-2003 Fredrick Meunier, Witold Filipczyk
+   Copyright (c) 2002-2004 Fredrick Meunier, Witold Filipczyk
 
-   $Id: scld.h,v 1.8 2003/11/18 13:27:39 pak21 Exp $
+   $Id: scld.h 2993 2007-06-17 13:54:49Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
@@ -117,26 +117,20 @@ extern scld scld_last_dec;           /* The last byte sent to Timex DEC port */
 
 extern libspectrum_byte scld_last_hsr; /* Last byte sent to Timex HSR port */
 
-extern libspectrum_byte timex_fake_bank[ 8192 ];
-
-extern memory_page timex_exrom_dock[8];
 extern memory_page timex_exrom[8];
 extern memory_page timex_dock[8];
-extern memory_page timex_home[8];
-extern memory_page timex_memory[8];
 
-void scld_reset( void );
+int scld_init( void );
+
 void scld_dec_write( libspectrum_word port, libspectrum_byte b );
-libspectrum_byte scld_dec_read( libspectrum_word port );
+libspectrum_byte scld_dec_read( libspectrum_word port, int *attached );
 
 void scld_hsr_write( libspectrum_word port, libspectrum_byte b );
-libspectrum_byte scld_hsr_read( libspectrum_word port );
+libspectrum_byte scld_hsr_read( libspectrum_word port, int *attached );
+
+void scld_memory_map( void );
 
 libspectrum_byte hires_get_attr( void );
 libspectrum_byte hires_convert_dec( libspectrum_byte attr );
 
-void scld_dock_free( void );
-void scld_exrom_free( void );
-void scld_home_free( void );
-
 #endif                  /* #ifndef FUSE_SCLD_H */
