@@ -1,7 +1,7 @@
 /* fbjoystick.c: Joystick emulation
    Copyright (c) 2003 Darren Salt
 
-   $Id: fbjoystick.c,v 1.1 2003/10/07 13:49:01 pak21 Exp $
+   $Id: fbjoystick.c 3096 2007-08-06 09:20:34Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
    Darren: linux@youmustbejoking.demon.co.uk
 
@@ -28,9 +27,4 @@
 
 #include <config.h>
 
-#ifdef UI_FB
-
 #include "../uijoystick.c"
-
-#endif				/* #ifdef UI_FB */
-
