@@ -1,7 +1,7 @@
 /* pixmaps.c: icon pixmaps
-   Copyright (c) 2003-2004 Fredrick Meunier, Philip Kendall
+   Copyright (c) 2003-2005 Fredrick Meunier, Philip Kendall, Darren Salt
 
-   $Id: pixmaps.c,v 1.2 2004/02/23 17:08:36 pak21 Exp $
+   $Id: pixmaps.c 2889 2007-05-26 17:45:08Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,17 +13,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
+#include <config.h>
+
 #include "gtkinternals.h"
 
 char *gtkpixmap_tape_inactive[] = {
@@ -92,6 +93,64 @@ char *gtkpixmap_tape_active[] = {
 ".+..>.................>..+.",
 "....>.................>...."};
 
+char *gtkpixmap_mdr_inactive[] = {
+"15 20 6 1",
+" 	c None",
+".	c #000000",
+"+	c #686868",
+"@	c #888888",
+"#	c #AFAFAF",
+"$	c #D8D8D8",
+"##+$$$$$$$$####",
+"###+++..+++####",
+"+##++++++++####",
+"$+##+++++++####",
+"$+#############",
+"$+@##@@@@@##@@#",
+"$+############@",
+"+#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"#@@@#######@@@#",
+"###############",
+"###############",
+"+++++++++++++++",
+"+++++++++++++++",
+"+++++++++++++++"};
+
+char *gtkpixmap_mdr_active[] = {
+"15 20 6 1",
+" 	c None",
+".	c #000000",
+"+	c #686868",
+"@	c #008800",
+"#	c #00AF00",
+"$	c #00D800",
+"##+$$$$$$$$####",
+"###+++..+++####",
+"+##++++++++####",
+"$+##+++++++####",
+"$+#############",
+"$+@##@@@@@##@@#",
+"$+############@",
+"+#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"@#############@",
+"#@@@#######@@@#",
+"###############",
+"###############",
+"+++++++++++++++",
+"+++++++++++++++",
+"+++++++++++++++"};
+
 char *gtkpixmap_disk_inactive[] = {
 "15 20 7 1",
 " 	c None",
@@ -234,3 +293,59 @@ char *gtkpixmap_tape_marker[] = {
 "...     ",
 "..      ",
 ".       "};
+
+char *gtkpixmap_mouse_inactive[] = {
+"14 21 4 1",
+"     c None",
+"#    c #6C6C6C",
+"+    c #A0A0A0",
+".    c #FFFFFF",
+"      ..      ",
+"      ..      ",
+" +####++####+ ",
+" #####++##### ",
+" ####++++#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ++++++++++++ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" +##########+ ",
+" +##########+ ",
+"  ++######++  "};
+
+char *gtkpixmap_mouse_active[] = {
+"14 21 4 1",
+"     c None",
+"#    c #0000D0",
+"+    c #6C6CD0",
+".    c #FFFFFF",
+"      ..      ",
+"      ..      ",
+" +####++####+ ",
+" #####++##### ",
+" ####++++#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ####+##+#### ",
+" ++++++++++++ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" ############ ",
+" +##########+ ",
+" +##########+ ",
+"  ++######++  "};
